/*********************************************************************
I2C.c                           (procedury obsugi I2C z przerwaniami)

Modu zawiera procedury obsugi ukadu I2C procesorw '214x/3x w 
trybie przerwa. Procedura i2c_init pozwala przygotowa ukad I2C do 
dziaania. Procedura i2c_engine rozpoczyna transmisj I2C a nastpnie
oczekuje na jej zakoczenie. Podprogram i2c_handler zawiera procedur
obsugi przerwa od ukadu I2C. Obsuga przerwania zaley od stanu 
zgoszonego przerwania. W kadym ze stanw programuje si przejcie
do kolejnego stanu.
*********************************************************************/
#include <LPC213x.H>
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "i2c.h"                      // prototypy funkcji obsugi I2C
                       
BYTE WR_NoBytes, RD_NoBytes;    // liczba bajtw do wysania i odbioru
BYTE Buff[40];                 // miejsce na nadawane i odbierane dane
DWORD Fin;                    // Fin>0 oznacza zakoczenie obsugi I2C

void i2c_init(void) { // przgotowanie I2C ----------------------------
  VICVectCntl1 = 0x20 | 9;         // przerwanie I2C (nr 9) w slocie 1
  VICVectAddr1 = (DWORD)i2c_handler;   // procedura obsugi przerwania
  VICIntEnable = 1 << 9;            // odblokuj  przerwanie I2C (nr 9)
  PINSEL0 |= 0x50;                // ustaw funkcj pinw GPIO jako I2C
  I2C0SCLH = 0x0F;                         //wybierz czstotliwo I2C
  I2C0SCLL = 0x0F;

  I2C0CONCLR = 0xFF;                             // zerowanie ustawie
  I2C0CONSET = 0x40;                                   //wczenie I2C
}

void i2c_engine(void) { // obsuga I2C -------------------------------
   I2C0CONSET = 0x20;                               // start I2C (STA)
   Fin=0;                                // znacznik koca obsugi I2C
   do {} while(  Fin==0 );          // czekanie na zakoczenie obsugi
}

void i2c_handler() __irq { // przerwania od I2C ----------------------
  static BYTE WR_indx,RD_indx ;

  switch(I2C0STAT) {                     //odczyt aktualnego stanu I2C

    case 0x08:                             // ustawiono warunek startu
	  I2C0DAT = Buff[0];     // wylij adres z bitem zapisu (R/~W = 0)
      I2C0CONCLR = 0x20;                     // zdejmij warunek startu
      WR_indx =0;
      RD_indx =0;
      break;
 
    case 0x10:                       // ustawiono ponowny start (STAr)
      I2C0CONCLR = 0x20;                     // zdejmij warunek startu
      I2C0DAT = Buff[WR_NoBytes+1];            // wylij adres z bitem
      break;                                   //...odczytu (R/~W = 1)
      
    case 0x18:        // otrzymano potwierdzenie adresu i trybu zapisu
      I2C0DAT = Buff[1];                   // wylij 1-szy bajt danych
      break;

    case 0x28:             // otrzymano potwierdzenie odebrania danych
      if (WR_indx < (WR_NoBytes-1)) {       // sprawdzenie ile wysano
           I2C0DAT = Buff[2+WR_indx];   //wylij kolejny bajt z bufora
           WR_indx++;                       // indeks wysanych bajtw
      } else {
      
           if (RD_NoBytes >0 )             // gdy s bajty do pobrania
              I2C0CONSET = 0x20;     // ponownie wylij warunek startu
           else {              // nie ma czego odbiera: RD_NoBytes =0
              I2C0CONSET = 0x10;                 //  wylij stop (STO)
              Fin =0x28;                             // koniec: Fin >0
           }
      }
      break;

    case 0x40:       // otrzymano potwierdzenie adresu i trybu odczytu
      if (RD_NoBytes==1) 
            I2C0CONCLR = 0x04;       // wyczamy potwierdzanie danych
      else     
            I2C0CONSET = 0x04;        // wczamy potwierdzanie danych
      break;

    case 0x50:                // otrzymano dane, wysano potwierdzenie
      if (RD_indx< (RD_NoBytes-1)) {
           Buff[WR_NoBytes+2+RD_indx]=I2C0DAT; // odczytaj bajt danych
      }
      if (RD_indx == (RD_NoBytes-2))              // przedostatni bajt
                               I2C0CONCLR = 0x04; // bez potwierdzenia
      RD_indx++;
      break;

    case 0x58:                    // otrzymano dane, bez potwierdzenia
      Buff [WR_NoBytes+RD_NoBytes+1] =I2C0DAT;  // ostatni bajt danych
      I2C0CONSET = 0x10;                       // ustaw warunek stopu 
      Fin=0x58;                                     // koniec: Fin > 0
      break;

    default:             // bad, zatrzymujemy i zaczynamy od pocztku
      I2C0CONSET = 0x10;                         //ustaw warunek stopu
      Fin=0xff;                                     // koniec: Fin > 0
      break;
  }
  I2C0CONCLR = 0x08;                     // zeruj flag przerwania I2C
  VICVectAddr = 0x00;                     // koniec obsugi przerwania
}