/***********************************************************************
LCD4bit.c                     (biblioteka LCD dla 4-bitowego interfejsu)

Biblioteka obsugi wywietlacza LCD podczonego do portu poprzez 4-linie danych.
Wywietlacz LCD podczony jest do portu P1:
   LCD          Procesor
   -------------------
   DB4        = P1.28
   DB5        = P1.29
   DB6        = P1.30
   DB7        = P1.31

   D/C~ (RS)  = P1.21        Data/Control~
   R/W~       = P1.22        Read/Write~
   E          = P1.23        Enable
************************************************************************/
#include <LPC214x.H>              // zbir definiujcy rejestry procesora
#include <LCD4bit.h>                                     // definicje LCD
#define INTERFACE LCD_4BIT 

// ------------------------  sprawdzenie flagi zajtoci wywietlacza LCD
char check_busy (void) {
    char bf=0;
    DC_LCD_clr;          // linia D/C~=0 : odczyt sowa Control (statusu)
    RW_LCD_set;                                  // linia R/W~=1 : odczyt
    E_LCD_set;                 // linia Enable E=1: odczyt z wywietlacza
    bf = DATA_from_LCD() & 0x08;        // najstarszy bit to zajto LCD
    E_LCD_clr;                         // linia Enable E=0:koniec odczytu
    E_LCD_set;                 // linia Enable E=1: odczyt z wywietlacza
    E_LCD_clr;   // linia Enable E=0:koniec odczytu drugiej czwrki bitw
    return (bf); 
}

//------------------------------------- zapis 4-bitw do wywietlacza LCD
void write_nibble (  char nibble) {
  E_LCD_clr;                           // Enable=0 wywietlacz nieaktywny
  RW_LCD_clr;  
  DATA_to_LCD(nibble);      //dane s przekazywane na liniach P0_3...P0_0
  E_LCD_set;                   // linia Enable E=1: zapis do wywietlacza
  E_LCD_clr;                           // linia Enable E=0: koniec zapisu
}

//------------------------------------- odczyt 4-bitw z wywietlacza LCD
char read_nibble (void) {
char nibble;
E_LCD_clr;                                                    // Enable=0
RW_LCD_set;                                             // odczyt: R/W~=1
E_LCD_set;                     // linia Enable E=1: odczyt z wywietlacza
nibble= DATA_from_LCD() ;                                // odczyt danych
E_LCD_clr;                            // linia Enable E=0: koniec odczytu
return(nibble);
}

//------------------- zapis sowa sterujcego Control do wywietlacza LCD
void LCD_ctrlWR ( char cmde) {
    unsigned var;
    WAIT;                           // sprawdzenie zajtoci wywietlacza
    DC_LCD_clr;                            // wybierz rejestr Control LCD
    write_nibble(cmde >> 4);      // zapis starszej czwrki bajtu Control
    write_nibble(cmde);           // zapis modszej czwrki bajtu Control
}

//------------------- odczyt sowa sterujcego Control z wywietlacza LCD
char LCD_ctrlRD ( void) {
  char ch;
  unsigned var;
  WAIT;                             // sprawdzenie zajtoci wywietlacza
  DC_LCD_clr;                              // wybierz rejestr Control LCD
  ch =read_nibble() <<4;         // odczyt starszej czwrki bajtu Control
  ch =ch | read_nibble() ;       // odczyt modszej czwrki bajtu Control
  return( ch);
}

//--------------------------------- zapis danych Data do wywietlacza LCD
void LCD_dataWR (char data) {
   unsigned var;
   WAIT; 
   DC_LCD_set;                         // wybierz rejestr danych Data LCD
   write_nibble(data >> 4);          // zapis starszej czwrki bajtu Data
   write_nibble(data);               // zapis modszej czwrki bajtu Data
}

//--------------------------------- odczyt danych Data z wywietlacza LCD
char LCD_dataRD ( void) {
 char byte;
 unsigned var;
WAIT;                               // sprawdzenie zajtoci wywietlacza
DC_LCD_set;                            // wybierz rejestr danych Data LCD
byte =read_nibble() <<4;            // odczyt starszej czwrki bajtu Data
byte =byte | read_nibble() ;        // odczyt modszej czwrki bajtu Data
return( byte);
}

#include "lcd_comm.c"                  // <<<<<<<<<<<<<<< wcz (include)