/*********************************************************************
timers_soft.c                       (uycie wielu ukadw odmierzania 
                                               ...sekund i milisekund)
Program pokazuje jak uywa jednoczenie wielu licznikw (timers)
odliczajcych czas. Liczniki odliczaj czas mierzony w 
milisekundach lub sekundach. Podstaw tego programu jest 
podprogram obsugi przerwania ukadu timer T0.
Rezultatu niezalenych licznikw to:

- co 1sek pojawi si cyfra od 0 do 9 na zczu UART
- co 10sek pojawi si napis "co 10sek. tekst"
- co 2sek  brzczyk wyda dwik przez 200msek
*********************************************************************/
#include <LPC214x.H>
#include <stdio.h>
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "UART.h"

#define Nbit 23  

#define Maxtimer   5                 // liczba czasomierzy  milisekund

#define T_message timer[1]  // czasomierz (timer) komunikatw- napisw
#define T_digit   timer[2]     // czasomierz (timer) wywietlania cyfr
#define T_buzzer  timer[3]             // czasomierz (timer) brzczyka
#define T_beep    timer[4]                 // czasomierz dugoci beep

#define  second      *1000
#define  msecond        *1

DWORD timer[Maxtimer];                         // zliczanie milisekund
WORD BlinkPeriod;
BYTE  pulse_bit=0;  
                                
// obsuga przerwa generowanych co 1 msek. --------------------------
void T0isr(void) __irq {

   static DWORD   Pulsecondount=0;
   WORD i;

   // modyfikacja licznikw odliczajcych milisekundy
   for(i=0;i<Maxtimer;i++) if(timer[i] > 0) timer[i]--;

   // ustalenie czasu generacji przebiegu okresowego (blinking)
   Pulsecondount++; 
   if((Pulsecondount > BlinkPeriod) && ( BlinkPeriod>0) ) {
       Pulsecondount=0;  
       pulse_bit=~pulse_bit;
   } 
   if ( BlinkPeriod==0) pulse_bit=0;

  T0IR        = 1;                 // zerowanie flagi przerwania od T0
  VICVectAddr = 0;                            // odblokowanie przerwa
}

// przygotowanie ukadu timer T0 -------------------------------------
void T0_init(void) {    
  T0PR= 15000-1;       // ustawienie preskalera: T0 mierzy milisekundy
  T0MR0 = 1;                          // odliczany czas to milisekunda

  T0MCR = 0x3;                                  // Interrupt and Reset
  T0TCR = 0x1;                                      // odblokowanie T0      
  VICVectAddr0 = (unsigned)T0isr;            // adres przerwania od T0
  VICVectCntl0 = 0x20 | 4;               // priorytet przerwania od T0
  VICIntEnable = 0x10;                  // odblokowanie przerwa od T0
}


// zerowanie wszystkich licznikw odliczajcych czasy ----------------
void reset_timers(void) { 
  WORD i;            
  for (i=0;i<Maxtimer;i++)   timer[i]= 0;       
}

// czekanie zadan liczb milisekund ---------------------------------
void delayMsec(DWORD val) {       
        timer[0]=val;
        while(timer[0]) {};           
}

//--------------------------------------------------------------------
void main(void) {  
    BYTE i=9;

    T0_init();                          // przygotowanie ukadu Timer0
    reset_timers();                       // zerowanie licznikw czasu
     UART_init;                          // 38400BPS (8-N-1) dla 24MHz                 
    BlinkPeriod=1; 

    IODIR1 |= 1<<Nbit;                        // dioda LED jako output
    IOCLR1 = 1<<Nbit;                        // dioda LED wieci przez
    delayMsec(1000 msecond);                     //... 1000msek.=1sek.
    IOSET1 = 1<<Nbit;                              // wycz diod LED
    while(1) {

      if (pulse_bit)  IOCLR1 = 1<<Nbit;         // zapalenie diody LED
      else            IOSET1 = 1<<Nbit;       //...zgaszenie diody LED
                                         //...zale od bitu pulse_bit

      if(T_buzzer==0) {  
         T_buzzer= 2 second;            // co 2sek uaktywni si Buzzer
         T_beep=200 msecond;                      // ... na czas 200ms
         BlinkPeriod=10 msecond;                     //okres: 2*10msek
      }
      if(T_beep==0) {                           // koniec impulsu beep
         BlinkPeriod=0; 
      }

      if(T_digit==0) {          
         T_digit= 1 second;           // co 1sek pojawi si nowa cyfra
         i=(i+1) % 10;
         putchar(0x30 + i);              // odliczanie w gr: 1...9,0
      }   

      if(T_message==0) {   
         T_message= 10 second;                             // co 10sek        
         puts("\r\n co 10sek. tekst \n\r"); //... pojawi si ten napis
      };
    }
}