/*********************************************************************
led_disp.c      (wywietlanie 4-cyfr na wywietlaczu LED przez Timer0)

Program wywietla zawarto zmiennej globalnej disp_val na 
wywietlaczu LED w postaci heksadecymalnej. Dynamiczne 
przeczanie niezbdna dla procesu wywietlania ukryto w 
podprogramie obsugi przerwania Timer0.
Aby przetestowa dziaanie programu w ptli gwnej (nmain)
wywietla si wszystkie moliwe liczby 16-bitowe w postaci
heksadecymalnej.
*********************************************************************/
#include <LPC214x.H>       
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

#define NBit_LEDS    16               // przesunicie dla 8-bitw kodu
#define NBit_PLACE   24         // przesunicie dla 4-bitw wybierania
#define M8           0xFF                        // maska dla 8- bitw
#define M4           0xF                          // maska dla 4-bitw

#define LEDS_init  IO1DIR |= IO1DIR | (M8 << NBit_LEDS )     // OUTPUT
#define PLACE_init IO1DIR |= IO1DIR | (M4 << NBit_PLACE )    // OUTPUT
          
// podprogram steruje wywietlaniem kodu  ----------------------------
inline void LEDS( BYTE x) {
  IO1CLR = ( ~x & M8 ) << NBit_LEDS;     // zerowanie wskazanych bitw
  IO1SET = (  x & M8 ) << NBit_LEDS;    // ustawianie wskazanych bitw
}

// podprogram steruje wyborem pozycji wywietlania -------------------
inline void PLACE( BYTE x) {
  IO1CLR = ( ~x & M4 ) << NBit_PLACE;    // zerowanie wskazanych bitw
  IO1SET = (  x & M4 ) << NBit_PLACE;   // ustawianie wskazanych bitw
}

DWORD timer_ms;
WORD disp_val=0x1234;            // warto pocztkowa do wywietlenia
 
// obsuga przerwa generowanych co 1 msek. --------------------------
void T0isr(void) __irq {

  static BYTE i;            // i okrela pozycj cyfry do wywietlania
  const digit[]= {       // definicje cyfr wywietlacza 8-segmentowego
                   0xc0,0xf9,0xa4,0xb0,               // cyfry 0,1,2,3
                   0x99,0x92,0x82,0xf8,               // cyfry 4,5,6,7
                   0x80,0x90,0x88,0x83,               // cyfry 8,9,A,b
                   0xC6,0xA1,0x86,0x8e                // cyfry c,d,E,F
  };  

              // wywietlanie zawartoci disp_val na wywietlaczu LEDS
  PLACE(0xff);
  LEDS(digit[ ( disp_val>>(4*i) ) & 0x0f ]);  // co wywietli na LEDS
  PLACE(~(1<<i));                          // gdzie wywietli na LEDS
  i= (i+1) & 0x03;
  
                     // modyfikacja licznika odliczajcego milisekundy
  if(timer_ms> 0) timer_ms--;
  T0IR        = 1;                 // zerowanie flagi przerwania od T0
  VICVectAddr = 0;                            // odblokowanie przerwa
}

// przygotowanie ukadu timer T0 -------------------------------------
void T0_init(void) {    
  T0PR= 15000-1;       // ustawienie preskalera: T0 mierzy milisekundy
  T0MR0 = 1;                          // odliczany czas to milisekunda

  T0MCR = 0x3;                                  // Interrupt and Reset
  T0TCR = 0x1;                                      // odblokowanie T0      
  VICVectAddr0 = (unsigned)T0isr;            // adres przerwania od T0
  VICVectCntl0 = 0x20 | 4;               // priorytet przerwania od T0
  VICIntEnable = 0x10;                  // odblokowanie przerwa od T0
}

// czekanie zadan liczb milisekund ---------------------------------
void delayMsec(DWORD val) {       
        timer_ms=val;
        while(timer_ms) {};           
}

//--------------------------------------------------------------------
void main(void) {  
  T0_init();                            // przygotowanie ukadu Timer0
  PLACE_init;                              // przygotowanie wyj GPIO
  LEDS_init;                               // przygotowanie wyj GPIO

     while(1) {  
         disp_val++;            // wylicz now liczb do wywietlania
         delayMsec(1000);                              // czekaj 1sek
    }
}