/*--------------------------------------------------------------------
 mtime.c                  (procedury zamiany daty i czasu na sekundy)

 Biblioteka moduw obsugi zegara i kalendarza typu LINUX. Procedury
 zamieniaj struktur TM opisujc dat i czas na sekundy liczone od
 1/01/1970. Licznik sekund to DWORD (4bajty).
--------------------------------------------------------------------*/
#include <LPC214x.H>   
#include <stdio.h>
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "mtime.h"

const WORD Days[12] = {                              // dni w miesicu
  31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31  };

const WORD YDays[12] = {                       // dni od pocztku roku
  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };

const char *SWeekday[7] = {
  "Sun", "Mon", "Tue", "Wed", "Thu", "Fri","Sat",  
};

// Zamiana czasu w sek. od 1970 do postaci struktury TM  -------------
void sec_to_TM( DWORD time, TM *tim ) {
    WORD hpery, i, cumdays;

    tim -> TM_sec = time % 60;
    time /= 60;                           // teraz time zawiera minuty
    tim -> TM_min = time % 60;
    time /= 60;                          // teraz time zawiera godziny
    i = time / (1461 * 24);            // liczba blokw po cztery lata
    tim -> TM_year  = (i << 2);               // blok * 4 = liczba lat
    tim -> TM_year += 70;
    cumdays = 1461 * i;
    time   %= 1461 * 24;  // reszta godzin po liczbie blokw co 4 lata

    while (1) {
      hpery = 365 * 24;                 // liczba godz. normalnym roku
      if ((tim -> TM_year & 3) == 0) hpery += 24;
      if (time < hpery) break;
      cumdays += hpery / 24;
      tim  -> TM_year++;
      time -= hpery;
    }               //teraz time zawiera liczb godzin w biecym roku

    tim -> TM_hour = time % 24;
    time /= 24;                        
    tim -> TM_doy = time +1;  // time zawiera liczb dni od pocz. roku
    cumdays += time + 4;                        // 1/1/1970:  czwartek
    tim -> TM_dow = (cumdays) % 7;                   // dzie tygodnia

    if ((tim -> TM_year & 3) == 0) {        // jeli to rok przestpny
        if (time > 59) time--;        // dzie pniejszy od 29 lutego
        else if (time == 59) {
               tim -> TM_month  = 2;                           // luty
               tim -> TM_dom = 29;                        // 29 lutego
               tim -> TM_year +=1900;
               return ; 
        }
    }

    for (tim -> TM_month = 0; 
                     Days[tim -> TM_month] <= time; tim -> TM_month++)
                                        time -= Days[tim -> TM_month];
    tim -> TM_dom = time + 1;
    tim -> TM_month += 1; 
    tim -> TM_year  +=1900;
}
 
// zamiana Daty i Czasu na sekundy od 1970  --------------------------
DWORD TM_to_sec (TM *tim )  {
    WORD year, month,day,hour,min,sec,leaps;
    DWORD days, secs;
    year = tim->TM_year -1900;                         // lata od 1900
    month= tim->TM_month-1;              // miesice liczone od 0...11
    day  = tim->TM_dom-1;         // dni w miesicu s liczone od 0...
    hour = tim->TM_hour;
    min  = tim->TM_min;
    sec  = tim->TM_sec;
    if (year < 70 || year > 138) return (0);                   // bd
   
    year -= 70;                                // lata liczone od 1970
    leaps = (year + 2) / 4;
    if (!((year+70) & 3) && (month < 2)) --leaps;
    days = year*365 + leaps + YDays[month] + day;
    secs = days*86400 + hour*3600 + min*60 + sec;
    return secs;
}

//zamia struktury TM na ASCII string dd/mm/yy hh:mm:ss ---------------
void  TM_to_string(TM *tim, char *s) {

  sprintf(s,"%s %2d/%2d/%4d   %2d:%2d:%2d     (%3d)  TM",
   SWeekday[tim-> TM_dow], tim->TM_dom,tim->TM_month,
    (tim->TM_year),tim->TM_hour,tim->TM_min,tim->TM_sec, tim->TM_doy);
}
