/***************************************************************************
noise_G.c   (generacja szumu przez generator liczb pseudolosowych)

Program generuje szum pseudolosowy dwoma sposobami. Pierwszy sposb to
symulacja rejestru o sprzniu liniowym typu "jeden-na-wiele"(typu Galois).
Drugi to uycie funkcji rand() ze standardowej biblioteki. Szum syszalny w
goniku doczonym do wyjcia DAC oraz moe by obserwowany na diodzie 
LED (po obnieniu czstotliwoci- opnienie czasowe). Przedstawiono 
przykady wielomianw 16- i 32- bitowych. Wielomiany zapisane s w odwrotnej
kolejnoci, poczynajc od bitu najmodszego. Czstotliwo prbkowania
sygnau wyjciowego wynosi ok. 300KHz. Okres powtarzania cigu 
pseudolosowego dla wielomianu 16-bitowego wynosi ok. 0.2sek a dla wielomianu
32-bitowego ok. 4godz. Osiem pierwszych prbek generowanego przebiegu jest
drukowane.
***************************************************************************/
#include <LPC214x.H>      
#include <stdlib.h>
#include <stdio.h>
#include "inttypes.h"                  // tu zdefiniowano BYTE, WORD i DWORD
#include "UART.h" 

//----------------------------------------------------- wielomiany 16-bitowe
#define   Polynom  0xA001 // CRC-16:    1+X^2+X^15+X^16      
//#define Polynom  0xD008 // XILINX:    1+X^1+X^3+X^12+X^16  
//#define Polynom  0x8408 // CRC-CCITT: 1+X^5+X^12+X^16      

//------------------------------------------------------ wielomian 32-bitowy
//  1+X^1+ X^2+ X^4+X^5+X^7+X^8+X^10+X^11+X^12+X^16+X^22+X^23+X^26+X^32 
//#define Polynom  0xedb88320                          // wielomian 32-bit  

//----------------------------------------------------- sterowanie diod LED
#define NBit_LED 23                          // dioda LED sterowana birem 16
#define LED_init IO1DIR |=  (1 << NBit_LED)               // bit jako OUTPUT
#define LED_clr  IO1CLR  =   1 << NBit_LED              // wyzeruj diod LED
#define LED_set  IO1SET  =   1 << NBit_LED                // wcz diod LED

//--------------------------------------------------------------------------
void main(void) {
 DWORD i,j=0;
 WORD CRC=0xFFFF;       // pocztkowy stan CRC (preset)dla liczb 16-bitowych
 //DWORD CRC=0xFFFFffff;     //pocztkowy stan (preset)dla liczb 32-bitowych

 PINSEL1 |= 1<<19;                               // wyjcie przetwornika DAC
 LED_init;  
 UART_init; 

 while(1) {
   //DACR= rand() & 0xFFc0;    // funkcja standardowa rand generuje 16-bitw
   DACR= ( CRC & 0xFFc0 ) ;                // wynik CRC syszalny w goniku
   if (CRC & 0x1) LED_set ; else LED_clr;                       // dioda LED
   if (CRC & 0x0001) CRC = (CRC  >> 1)^ Polynom  ;
   else  CRC = (CRC>> 1)  ; 
   //for (i=0;i<0x10000;i++);                    // opnienie dla diody LED
   //for (i=0;i<0x100;i++);         // opnienie dla wielomianu 16-bitowego
   if (j++<8 )printf(" %8x \n", CRC);     // drukowanie 8-miu prbek sygnau
 }
}
