/***************************************************************************
circ_intr.c                 (biblioteka moduw obsugi UART z przerwaniami)
                           .                         ... i buforami koowymi

Najwaniejsze procedury tej biblioteki to _getkey() i putchar().Bdy 
nadawania lub odbioru powstaj gdy TXByteCount > TXBUFSIZE lub 
RXByteCount > RXBUFSIZE. Wywoanie   if(RXByteCount > 0) _getkey()
zabezpiecza przed utkniciem w ptli czekania na bajt. Bufor FIFO podczas
sterownia lini kierunku DIR485 musi by wyczony.
***************************************************************************/
#include <LPC214x.H>      
#include "inttypes.h"                  // tu zdefiniowano BYTE, WORD i DWORD

// pin DIR485_SEL jest sterowany poprzez port GPIO -------------------------
#define DIR485_SEL    1<<10                              // numer bitu P0.10
#define DIR485_set    IOSET0 =  DIR485_SEL
#define DIR485_clr    IOCLR0 =  DIR485_SEL
#define DIR485_init   IODIR0 |= DIR485_SEL

#define RXBUFSIZE 256                    // rozmiar bufora odebranych bajtw
BYTE RXBuf[RXBUFSIZE];                            // bufor odebranych bajtw
WORD RXIndexRD, RXIndexWR;               // indeksy bufora odebranych bajtw
WORD RXByteCount;                               // licznik odebranych bajtw
#define TXBUFSIZE 256                    // rozmiar bufora nadawanych bajtw
BYTE TXBuf[TXBUFSIZE];                            // bufor nadawanych bajtw
WORD TXIndexRD, TXIndexWR;               // indeksy bufora nadawanych bajtw
WORD TXByteCount;                               // licznik bajtw do nadania
BYTE TXBufEmpty;          // znacznik gdy bufor nadawanych bajtw jest pusty

#define RXE U0IER |=  0x1                                        //enable Rx
#define RXD U0IER &= ~0x1                                       //disable Rx

#define TXE U0IER |=  0x2                                        //enable Tx
#define TXD U0IER &= ~0x2                                       //disable Tx


void UART_irq (void) __irq ;          // prototyp funkcji obsugi przerwania

// ustalenie prdkoci transmisji ------------------------------------------
void com_baudrate (WORD baudrate) {
  unsigned long DLreload;

  DLreload = 15000000UL / baudrate / 16UL;
  U0LCR |=  0x80;            // ustaw DLAB aby dokona wpisu do U0DLL, U0DLM
  U0DLL  =  DLreload;
  U0DLM  = (DLreload >> 8);
  U0LCR &= ~0x80;                                            // wyzeruj DLAB
}

// przygotownie transmisji UART --------------------------------------------
void UART_init (void) {
  RXIndexWR = RXIndexRD = RXByteCount = 0;
  TXIndexWR = TXIndexRD = TXByteCount = 0;
  TXBufEmpty = 1;                                     // bufor TX jest pusty
  DIR485_init;
  DIR485_clr;                                        // kierunek 485: odbir

  PINSEL0 = 0x5;    // obsuga RxD0, TxD0 UART0  (0x50000 dla RxD1 and TxD1)
  U0LCR = 0x03;                                                     // 8-N-1
  U0IER = 0;                               // zablokowanie przerwa od UART0
  VICVectAddr0 = (WORD) UART_irq;
  VICVectCntl0 = 0x20 | 6;       // UART0 Interrupt (7 for  UART1 Interrupt)
  VICIntEnable = 1 << 6;  // Enable UART0 Interrupt (7 for  UART1 Interrupt)
  com_baudrate (38400);                               // prdko transmisji

  U0IIR;             // odczyt IrqID pozwala przygotowa nastpne przerwanie
  U0IER = 3;                                // UART1: RX and THRE Interrupts
}

// obsuga przerwania UART -------------------------------------------------
void UART_irq (void) __irq  {                               // Serial Port 0
  volatile char IIR;

  while (((IIR = U0IIR) & 0x01) == 0) {     // powtarzanie gdy s przerwania
     switch (IIR & 0x0E) {
       case 0x06:                                         // bd odbioru RX
            U0LSR;                                          // zignoruj bd
            break;

       case 0x04:                                  // odebrane dane dostpne
       case 0x0C:                                 // obir danych i time-out
            RXBuf[RXIndexWR++] = U0RBR;        // przelij bajt do bufora RX
            RXIndexWR &= RXBUFSIZE-1;                  // operacja cykliczna
            RXByteCount++;                      // licznik odebranych bajtw
            break;

       case 0x02:                       // dane wysane: podaj nastpny bajt

            DIR485_clr;                              // kierunek 485: odbir
            if (TXByteCount)  {
                  DIR485_set;                     // kierunek 485: nadawanie
                  U0THR = TXBuf[TXIndexRD++];    // kolejny bajt do wysania
                  TXIndexRD &= TXBUFSIZE-1;
                  TXByteCount--;
            }
            else   TXBufEmpty = 1;         // gdy nie ma ju nic do wysania
            break;

       case 0x00:                                    // przerwanie od modemu
            U0MSR;                                          // zignoruj bd
            break;

       default:
            break;
     }
  }
  VICVectAddr = 0;                                  // odblokowanie przerwa
}

// podprogram putchar zgodny z stdio.h -------------------------------------
// Zapis bajtu do bufora TX. Gdy bufor TX jest pusty wysanie bajtu
// bezporednio. Reszta bajtw z bufora TX wysya podprogram 
// przerwania UART w sposb automatyczny.
int putchar (int Byte) {

   while((TXByteCount>=(RXBUFSIZE-1) ) && !TXBufEmpty ) {};        // czekaj
   TXD;                                          // blokowanie przerwa UART
   TXBuf[TXIndexWR++] = Byte;                    // zapisz bajt do bufora TX
   TXIndexWR &= TXBUFSIZE-1;                      // obsuga bufora koowego
   TXByteCount++;                  // zwikszenie licznika znakw do nadania
   TXE;                                        // odblokowanie przerwa UART
   if (TXBufEmpty && TXByteCount>0 ) {        // bufor TX zawiera 1-szy bajt
        TXD;                                     // blokowanie przerwa UART
        TXBufEmpty = 0;                       // bufor zawiera pierwszy bajt
        DIR485_set;                               // kierunek 485: nadawanie
        U0THR = TXBuf[TXIndexRD++];                           // wylij bajt
        TXIndexRD &= TXBUFSIZE-1;                 // obsuga bufora koowego
        TXByteCount--;            // zmniejszenie licznika znakw do nadania
        TXE;                                   // odblokowanie przerwa UART
   }
   return (Byte);                      // tak zdefiniowano putchar w stdio.h
}

// podprogram _getkey zgodny z stdio.h -------------------------------------
// podprogram pobiera kolejny bajt z bufora RX
int _getkey (void) {
  char Byte;
  while (RXByteCount==0) {};          // czekanie na bajt gdy bufor RX pusty
  RXD;                                           // blokowanie przerwa UART
  Byte = RXBuf[RXIndexRD++];                   // pobranie bajtu z bufora RX
  RXIndexRD &= RXBUFSIZE-1;                   // operacja na buforze koowym
  RXByteCount--;                    // poprawienie licznika znakw pobranych
  RXE;                                         // odblokowanie przerwa UART
  return (Byte);                                          
}

// kasowanie bufora RX -----------------------------------------------------
void flush_RX(void) {
    RXD;                                         // blokowanie przerwa UART
    RXByteCount = 0; RXIndexRD = 0;                   // indeksy na pocztek
    RXE;                                       // odblokowanie przerwa UART
}

// liczba znakw w buforze RXCount -----------------------------------------
BYTE  RX_Count(void) {
   return(RXByteCount);
}