/*********************************************************************
thermometer.c                                       (termometr 1-wire)

Program odczytuje temperatur z ukadu DS1820. Obsuga termometru 
odbywa si poprzez 1-przewodowy interfejs zwany 1-wire. Procesory 
'214x nie posiadaj kontrolera urzdze typu 1-wire. Osuga interfejsu
jest zrealizowana software'owo. 
Wyniki oblicze przesyane s na zcze SIO: 38400bps (8-N-1).
*********************************************************************/
#include <LPC214x.H>                    // LPC214x definicje rejestrw
#include <stdio.H>  
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

#include "1_wire.H"                       // obsuga interfejsu 1-wire
#include "UART.H"                          // obsuga transmisji RS232

//--------------------------------------------------------------------
void main(void) {
   char buffer[9];                          // bufor odebranych bajtw
   int i;

   UART_init;                     // inicjowanie transmisji szeregowej
   printf("\r\nWaiting for a 1-WIRE device...\n\r"); // napis startowy
   while (1) {
     while(!WIRE1_present()); //sprawdzenie odpowiedzi na 1-wire reset
     WIRE1_write_byte(0x33);    // polecenie: podaj kod identyfikatora
     printf("\n\r ID   : ");
     for(i=0;i<8;i++) printf("%02X ",buffer[i]=WIRE1_read_byte());
     printf("\n\r");
     break;   
   }  

   while (1) {
     while(!WIRE1_present()); //sprawdzenie odpowiedzi na 1-wire reset
     if(WIRE1_present()) {
        WIRE1_write_byte(0xcc); // polecenie: jeden termometr na linii
        WIRE1_write_byte(0x44);       // polecenie: zmierz temperatur
        printf(" Start: ");
     } 

     while( WIRE1_read_byte() ==0 ) ;     // odpowied na 1-wire reset
     if(WIRE1_present()) {
        WIRE1_write_byte(0xcc); // polecenie: jeden termometr na linii
        WIRE1_write_byte(0xbe);             // odczytaj RAM termometru
        for(i=0;i<9;i++) printf("%02X ", buffer[i]=WIRE1_read_byte());
        printf(" Temp = %6.1f \r", (buffer[0]  +( buffer[1]<<8))*0.5);
        delay_us(1000000);
     }
   }
}