/*********************************************************************
RTL.c                          (procedury obsugi transmisji Ethrenet)

Przykadowe procedury obsugi kontrolera Ethernet typu RTP8019AS.
Procedury pozwalaj zainicjowa, wysa i odebra ramk Ethernet. Aby
mona byo pokaza praktyczne efekty dziaania  wysyane i odbierane
ramki to ARP (Address Resolution Packet). Ramki ARP maj prost budow
i ich przesanie poprzedza standardowe zapytanie PING. Procedura
odbioru ramek jest bardzo uproszczona i zakad odbir ramek nie
duszych ni 256bajtw. Ramki ARP speniaj to kryterium bo ich
dugo to 64bajty.
*********************************************************************/
#include <LPC214x.H>       

#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "UART.h"

#include "RTLhardware.h"                   // podstawowe procedury MAC
#include "RTLregs.h"                             // rejestry RTL8019AS
#include "RTL.h"                  // inicjowanie, zapis i odczyt ramki

#define PAGE_SIZE       0x100                  // strona to 256 bajtw
#define START_PAGE      0x40                       // pocztek  bufora
#define STOP_RX_PAGE    0x60                          // koniec bufora
#define TX_PAGES        6          // maksymalny rozmiar ramki 6 * 256
#define TX_BUFFERS      2                            // liczba buforw
#define FIRST_TX_PAGE   START_PAGE                 // bufor nadawania 
#define FIRST_RX_PAGE   (FIRST_TX_PAGE+TX_PAGES*TX_BUFFERS)  // odbir

extern  BYTE ARP[28];                                       // ramka ARP

BYTE NICReset(void) { //------------------------------ reset RTL8019AS

    BYTE i;
    NICStart();

    for (i = 0; i < 20; i++) {
        Delay_us(500);
        NICPut(RESET,  NICGet(RESET));
        if ((NICGet(P0_ISR) & 0x80) )    return 0;   // wykonano reset                
    }  
    return 1;  // reset nie powid si
}

BYTE NICInit(void) {  //------------------------ inicjowanie RTL8019AS

    BYTE i;   

    if (NICReset()) return 1;
    NICPut(CMD, 0x21);                              // MAC soft restet
    Delay_us(1600);
                                        // Data Configuration Register
    NICPut(P0_DCR, 0x48);                // 8-bitw, 2 razy 16-bit DMA
                                        // Remote Byte Count Registers
    NICPut(P0_RBCR0, 0);                                  // zerowanie
    NICPut(P0_RBCR1, 0);
                                     // Receive Configuration Register
    NICPut(P0_RCR, 0x00);           // tylko obir ramek z adresem MAC
                                             // MAC in Loopback Mode 1
    NICPut(P0_TCR, 0x02);                       // internal loopback 1
                                     // Transmit Buffer Start Register
    NICPut(P0_TPSR, FIRST_TX_PAGE);
                                     // Initialize Receive Buffer Ring
    NICPut(P0_PSTART, FIRST_RX_PAGE);
    NICPut(P0_PSTOP, STOP_RX_PAGE);   
    NICPut(P0_BNRY, STOP_RX_PAGE - 1);
                                          // Interrupt status register
    NICPut(P0_ISR, 0xff);                // kasuj wszystkie przerwania
                                            // Interrupt Mask Register
    NICPut(P0_IMR, 0x00);                // brak przerwa zewntrznych
    NICPut(CMD, 0x61);                       // Select Page 1 and Stop
                                         // Physical Address Registers
    for (i = 0; i < 6; i++)  NICPut(P1_PAR0 + i, ARP[i+8]); // mj MAC        
                                                // Multicast Registers
    for (i = 0; i < 8; i++) NICPut(P1_MAR0 + i, 0);
                                                    // Current Pointer
    NICPut(P1_CURR, FIRST_RX_PAGE);
    NICPut(CMD, 0x21);                       // Select Page 0 and Stop
                                          // Transmit Control Register
    NICPut(P0_TCR, 0);                 // loopback off:Set Normal Mode
                                               // Activate transmitter
    NICPut(CMD, 0x22);               // complete remote DMA: NIC start
    return 0;
}

void NICSend(BYTE cnt) { //-------------------------wysanie ramki ARP
    
    BYTE length =60;          // minimalna ramka to 64 bajty minus CRC
    BYTE i   ;                 
                                                  // Remote Byte Count
    NICPut(P0_RBCR0, length);             // liczba bajtw do wysania
    NICPut(P0_RBCR1, length >> 8);
                                                  // Remote Start Addr
    NICPut(P0_RSAR0, 0);                   
    NICPut(P0_RSAR1, FIRST_TX_PAGE);      // adres ramki do transmisji

    NICPut(CMD, 0x12);      // zapis ramki do bufora: Remote DMA Write

    //  nagowek Ethernet ============================================
    for (i = 0; i < 6; i++)  NICPut(IOPORT, 0xFF);    // MAC Broadcast
    for (i = 0; i < 6; i++)  NICPut(IOPORT, ARP[i+8]);       // my MAC

    NICPut(IOPORT, 0x08);                        // znacznik ramki ARP
    NICPut(IOPORT, 0x06);    

    //dane Ethernet ==================================================
    for (i = 0; i < 27; i++)  NICPut(IOPORT, ARP[i]);     // ARP frame
    NICPut(IOPORT,cnt );                      // skanowanie adresw IP
    for (i = 0; i < length-28; i++) NICPut(IOPORT, 0);  // dopenienie
                  
    NICPut(CMD, 0x22);        // koniec zapisu do bufora: Complete DMA
    for (i = 0; i < 20; i++)
     if (NICGet(P0_ISR) & 0x40) break; //czekanie na koniec transmisji
    NICPut(P0_ISR, 0x40);     // zerowanie zgoszenia koca transmisji
                            
    NICPut(P0_TBCR0, (length  & 0xff));               // dugo ramki
    NICPut(P0_TBCR1, ((length >> 8) & 0xff));

    NICPut(P0_TPSR, FIRST_TX_PAGE);       // adres ramki do transmisji
    NICPut(CMD, 0x26);                            // transmituj pakiet
}

BYTE NICReceive(WORD len, BYTE *frame) { // odbir ramki ARP ---------

    BYTE wr,rd;
    DWORD i;

    for (i = 0; i < 500; i++)
      if (NICGet(P0_ISR) & 0x01) goto ok;  // czekanie na odbir ramki
    return 1;

ok: NICPut(P0_ISR, 0x01);               // kasowanie znacznika odbioru
    NICPut(CMD, 0x60);                           // wybr strony page1
    wr=NICGet(P1_CURR);           // miejsce do zapisu nastpnej ramki
    NICPut(CMD, 0x20);                           // wybr strony page0

    NICPut(P0_BNRY,wr);
    rd=wr-1;
    if (rd== FIRST_RX_PAGE-1) rd=STOP_RX_PAGE - 1; 
	NICPut(P0_RSAR0, 0);    
    NICPut(P0_RSAR1, rd);	      // miejsce gdzie jest ostatnia ramka
    
    NICPut(P0_RBCR0, len & 0xFF);          // liczba bajtw do odczytu
    NICPut(P0_RBCR1, len >>8);
    NICPut(CMD, 0x0a);                         // odczytaj blok danych

    for (i=0; i<len; i++ ) *frame++ = NICGet(IOPORT); // kolejne bajty

    return 0;             
}