/*********************************************************************
RTLhardware.c                          (procedury niskiego poziomu ...
                                  podczenia RTL8019AS  do procesora)

Procedury niskiego poziomu, pozwalajce na obsug ukadu RTL8019AS. 
Ukad RTL posiada magistral:8 linii danych, 5 linii adresu oraz linie
sterujce RD, WR, CS i RESET. Takiej magistrali nie posiada procesor
'214x. Za pomoc procedur poniej symuluje si dziaanie magistrali
ukadu RTLsterujc bitami portw GPIO. 
*********************************************************************/
#include <LPC214x.H>       
#include "RTLhardware.h"

#define NBit_Data     24        // przesunicie dla 8-miu bitw danych
#define NBit_Addr      4        // przesunicie dla 5-ciu bitw adresu
#define NBit_reset    30                       // pooenie bitu RESET
#define NBit_cs       20                          // pooenie bitu CS
#define NBit_rd       21                          // pooenie bitu RD
#define NBit_wr       22                          // pooenie bitu WR
                                      
#define M           (0xFFu)            // maska dla 8-miu bitw danych
#define N           (0x1Fu)            // maska dla 5-ciu bitw adresu

__inline void data_to_NIC()   { IO1DIR |= M << NBit_Data ;  }   // OUT
__inline void data_from_NIC() { IO1DIR &= ~(M << NBit_Data);}    // IN

__inline void NIC_DATA_put(BYTE x) {                   // zapis danych

  IO1CLR = ( ~x & M ) <<  NBit_Data  ; 
  IO1SET = (  x & M ) <<  NBit_Data  ; 
}

__inline BYTE  NIC_DATA_get (void ) {                 // odczyt danych
  return( (IO1PIN & ( M<< NBit_Data  ) ) >> NBit_Data);  
}  

__inline void addr_to_NIC()        { IO0DIR |= N << NBit_Addr;} 
__inline void NIC_Addr_clr(BYTE x) {IO0CLR  = ( ~x & N) << NBit_Addr;}             
__inline void NIC_Addr_set(BYTE x) {IO0SET  = (  x & N) << NBit_Addr;}             
__inline void NIC_ADDR_put(BYTE x) {NIC_Addr_clr(x); NIC_Addr_set(x);}

__inline void setb(BYTE b)   {IO0SET  =  1 << (b);  } 
__inline void clrb(BYTE b)   {IO0CLR  =  1 << (b);  }
__inline void dirb(BYTE b)   {IO0DIR |=  1 << (b);  }

#define NIC_RESET_out   dirb( NBit_reset)                  // kierunek
#define NIC_RESET_set   setb( NBit_reset)                     // ustaw
#define NIC_RESET_clr   clrb( NBit_reset)                     // zeruj

#define NIC_CS_out      dirb( NBit_cs ) 
#define NIC_CS_set      setb( NBit_cs )           
#define NIC_CS_clr      clrb( NBit_cs )         

#define NIC_RD_out      dirb( NBit_rd ) 
#define NIC_RD_set      setb( NBit_rd )        
#define NIC_RD_clr      clrb( NBit_rd )          

#define NIC_WR_out      dirb( NBit_wr ) 
#define NIC_WR_set      setb( NBit_wr )           
#define NIC_WR_clr      clrb( NBit_wr )       

#define DATA_TO_NIC     data_to_NIC() 
#define DATA_FROM_NIC   data_from_NIC()

#define ADDR_TO_NIC     addr_to_NIC() 

void NICStart(void) { //---------------------------------przygotowanie
    DATA_FROM_NIC;
    NIC_ADDR_put ( 0);
    ADDR_TO_NIC ;

    NIC_CS_clr;                           // ukad zawsze wybrany CS=0
    NIC_WR_set;                                // stan pocztkowy WR=1
    NIC_RD_set;                                // stan pocztkowy RD=1
    NIC_RESET_set;
    NIC_WR_out;
    NIC_RD_out;
    NIC_CS_out;
    NIC_RESET_out;
    Delay_us(2000);
    NIC_RESET_clr;
    Delay_us(2000);                     // generowanie RESET RTL8019AS
}

void NICPut(BYTE reg, BYTE val) { //----------------------zapis do NIC
    NIC_ADDR_put (reg);
    NIC_DATA_put ( val);
    DATA_TO_NIC;
    NIC_WR_clr ;
    Delay_us(1);                                 // krtki impuls WR=0
    NIC_WR_set ;
    DATA_FROM_NIC;                                  // kierunek odczyt
}

BYTE NICGet(BYTE reg) { //--------------------------------odczyt z NIC
    BYTE val;
    DATA_FROM_NIC;
    NIC_ADDR_put (reg);
    NIC_RD_clr;                                         // odczyt RD=0
    val = NIC_DATA_get();                            // odczytane dane
    NIC_RD_set;
    return(val);
}

void Delay_us(DWORD us) { //-------------------------opnienie w usek
   DWORD i;
   for(i=0;i< (long)(5*us);i++){}; 
}