/********************************************************************
LCD_4bit_test.c       (Wywietlacz sterowany przez 4 linie danych
                               - modsza cze portu P0 zawiera dane)

Program demonstruje sposb obsugi wywietlacza LCD podczonego do
portu poprzez 4-linie danych, do mdszej czci portu P0.
********************************************************************/
#include <LPC214x.H>       // zbir definiujcy rejestry procesora

void LCD_ctrlWR ( char cmde);
void LCD_dataWR ( char byte);
void LCD_ini (void);
char check_busy (void);

#define WAIT  for(var=0; var<0x6000;var++)           // (1) !
//#define WAIT  while (check_busy() )                // (2) !

//-------------------------------------------------------------------
main(void) {
  char i;
  LCD_ini();

  for(i=0;i<16;i++) {
       LCD_dataWR('0'+i);
  }

  while(1);
}

/*====================================================================
Wywietlacz LCD podczony jest do portu P0:
   LCD          uP
   -------------------
   DB4        = P1.28
   DB5        = P1.29
   DB6        = P1.30
   DB7        = P1.31

   D/C~ (RS)  = P1.21     Data/Control~
   R/W~       = P1.22     Read/Write~
   E          = P1.23     Enable
====================================================================*/
#define DATA_LCD_NBit 28                                                                                                                                                                                // wywietlacz podczony do portu P0  
#define DC_LCD_NBit   21                  // linia D/C~ (Data/Control)      
#define RW_LCD_NBit   22 			        // linia R/W~ (Read/Write) 
#define E_LCD_NBit    23                           // linia E (Enable) 
#define M4            0xFU 

#define E_LCD_clr    IO1CLR = 1 << E_LCD_NBit  // wyzeruj E
#define E_LCD_set    IO1SET = 1 << E_LCD_NBit  // ustaw E
#define DC_LCD_clr   IO1CLR = 1 << DC_LCD_NBit // wyzeruj DC (control)
#define DC_LCD_set   IO1SET = 1 << DC_LCD_NBit // ustaw DC	 (data)
#define RW_LCD_clr   IO1CLR = 1 << RW_LCD_NBit // wyzeruj RW (control)
#define RW_LCD_set   IO1SET = 1 << RW_LCD_NBit // ustaw RW   (data)

__inline void  DATA_to_LCD(char x)  { 
   IO1CLR = ( (~x) & M4 ) << DATA_LCD_NBit ;
   IO1SET = ( ( x) & M4 ) << DATA_LCD_NBit ;  
}

__inline char  DATA_from_LCD(void)  {
 char x;
 IO1DIR = IO1DIR & (~( M4<< DATA_LCD_NBit )) ;
 x=	(IO1PIN & ( M4<< DATA_LCD_NBit ) ) >>DATA_LCD_NBit;
 IO1DIR = IO1DIR | ( M4<< DATA_LCD_NBit ) ;
 return  ( x); 
}

__inline void  LCD_all_init (void) {
  IO1DIR = IO1DIR |  ( M4 <<  DATA_LCD_NBit )  | (1 << E_LCD_NBit ) |
	      (1 << DC_LCD_NBit) |  (1 << RW_LCD_NBit);

}	
//--------------------------------- zapis 4-bitw do wywietlacza LCD 
void write_nibble (  char nibble) {
  E_LCD_clr;                       // Enable=1 wywietlacz nieaktywny 
  RW_LCD_clr;  
  DATA_to_LCD(nibble);  //dane s przekazywane na liniach P1.3...P1.0
  E_LCD_set;               // linia Enable E=0: zapis do wywietlacza  
  E_LCD_clr;                       // linia Enable E=1: koniec zapisu  
}


//--------------- zapis sowa sterujcego Control do wywietlacza LCD 
void LCD_ctrlWR (  char cmde) {
    unsigned var;
    WAIT;                       // sprawdzenie zajtoci wywietlacza  
    DC_LCD_clr;                        // wybierz rejestr Control LCD  
    write_nibble(cmde >> 4);  // zapis starszej czwrki bajtu Control 
    write_nibble(cmde);       // zapis modszej czwrki bajtu Control 
}


//----------------------------- zapis danych Data do wywietlacza LCD 
void LCD_dataWR (char data) {
   unsigned var;
   WAIT; 
   DC_LCD_set;                     // wybierz rejestr danych Data LCD  
   write_nibble(data >> 4);      // zapis starszej czwrki bajtu Data 
   write_nibble(data);           // zapis modszej czwrki bajtu Data 
}


//-----------------------------------  inicjalizacja wywietlacza LCD
void LCD_ini (void) {
char init_tab[] = {0x33,0x32,0x28,0x01, 0x0F};       // inicjalizacja
unsigned i,j;
 for (j = 0; j <0x1fffff; j++) ; 
  LCD_all_init();
  for (i = 0; i < sizeof init_tab; i++) {
       LCD_ctrlWR(init_tab[i]);          // zapis danych inicjujcych
 }
}

// --------------------- sprawdzenie flagi zajtoci wywietlacza LCD
char check_busy (void) {
    char bf;
    DC_LCD_clr;      // linia D/C~=0 : odczyt s-owa Control (statusu)  
    RW_LCD_set;                              // linia R/W~=1 : odczyt  
    E_LCD_set; 
    bf = DATA_from_LCD() & 0x08;    // najstarszy bit to zajto LCD 
    E_LCD_clr; 
    E_LCD_set;             // linia Enable E=0: odczyt z wywietlacza  
    E_LCD_set;  
	E_LCD_clr;       
	return (bf);
}

