// -----------------------------------------------------------------------
// zbir : LCD.h
// Zbir zawiera definicje umoliwiajce uycie znakowego wywietlacza LCD
// Przed yciem wywietlacza LCD naley wywoa makrodefinicj LCD_init
// -----------------------------------------------------------------------

#ifndef _LCD_H_
#define _LCD_H_

#define WAIT  for(var=0; var<0x5000;var++)                        // (1) !
//#define WAIT  while (check_busy()) // sprawdzenie linii BUSY w LCD (2) !

#define NB_ROW 02                         // liczba linii wywietlacza LCD
#define NB_COL 16                // liczba znakw w linii wywietlacza LCD

//Polecenia sterujce do LCD ---------------------------------------------
#define ROW0 0x00                       // adres wiersza nr 0 wywietlacza
#define ROW1 0x40                       // adres wiersza nr 1 wywietlacza
#define ROW2 0x00+NB_COL                // adres wiersza nr 2 wywietlacza
#define ROW3 0x40+NB_COL                // adres wiersza nr 3 wywietlacza
#define CG_RAM   0x40   // polecenie zapisu do pamici definicji znakw CG
#define DD_RAM   0x80             // polecenie zapisu do pamici danych DD
#define CLR_LCD  0x01                                 // zeruj wywietlacz
#define CUR_ON   0x0F                                      // wcz kursor
#define CUR_OFF  0x0C                                     // wycz kursor
#define LCD_4BIT 0x28                                // 4-bitowy interfejs
#define LCD_8BIT 0x38                                // 8-bitowy interfejs

#define DATA_LCD_NBit 24  //wywietlacz podczony do portu P1, od bitu 24
                                                                                   
#define DC_LCD_NBit   21                      // linia D/C~ (Data/Control)
#define RW_LCD_NBit   22                        // linia R/W~ (Read/Write)
#define E_LCD_NBit    23                               // linia E (Enable)
#define M            0xFFu                             //maska dla 4-bitw

#define E_LCD_clr    IO1CLR = 1 << E_LCD_NBit    //              wyzeruj E
#define E_LCD_set    IO1SET = 1 << E_LCD_NBit    //                ustaw E
#define DC_LCD_clr   IO1CLR = 1 << DC_LCD_NBit   // wyzeruj DC   (control)
#define DC_LCD_set   IO1SET = 1 << DC_LCD_NBit   // ustaw DC        (data)
#define RW_LCD_clr   IO1CLR = 1 << RW_LCD_NBit   // wyzeruj RW   (control)
#define RW_LCD_set   IO1SET = 1 << RW_LCD_NBit   // ustaw RW        (data)

__inline void  DATA_to_LCD(char x)  { //------------------------------------
   IO1CLR = ( (~x) & M ) << DATA_LCD_NBit ;
   IO1SET = ( ( x) & M ) << DATA_LCD_NBit ;  
}

__inline char  DATA_from_LCD(void)  { //------------------------------------
 char x;
 IO1DIR &= ~( M<< DATA_LCD_NBit ) ;
 x       = (IO1PIN & ( M<< DATA_LCD_NBit ) ) >>DATA_LCD_NBit;
 IO1DIR  = IO1DIR | ( M<< DATA_LCD_NBit ) ;
 return  ( x); 
}

__inline void  LCD_all_init (void) { //-------------------------------------
      IO1DIR |= ( M <<  DATA_LCD_NBit )  | (1 << E_LCD_NBit ) |
                (1 << DC_LCD_NBit)        | (1 << RW_LCD_NBit);
}   
                                 
char check_busy (void);
void lcd_init (void);                  // inicjacja wywietlacza - funkcja
void LCD_clear (void);                           // zerowanie wywietlacza
void LCD_cursorOFF (void);                          // wyczenie kursora:
                                      //... domylnie kursor jest wczony
                                 // ustaw pozycj kursora: wiersz, kolumna
void LCD_set_cursor (char rownr, char  colnr);         // numeracja od 0,0
#define LCD_init  lcd_init ()            // inicjacja wywietlacza - makro
#endif // _LCD_H_ 
