/*********************************************************************
spi1_hard.c                                    (ukadowa obsuga SPI1)

Biblioteka moduw obsugi SPI1 metod ukadow w procesorach '214x.
Kady z podprogramw jednoczenie zapisuje i odczytuje dane. Modu
SPI_Byte - nadaje i odbiera bajt danych
SPI_Data - nadaje i odbiera cig bajtw.
*********************************************************************/
#include <LPC214X.H> 
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

//-------------------------------------------------- inicjalizacja SPI
void SPI_Init(void) {
    BYTE i;
    PINSEL1 |=  0x00A8;                    // przygotowanie pinw SPI1
    SSPCR1 = 0x00;           // zablokuj SPI1 aby ustawi pejestry SSP
    SSPCR0 = 0x0707;   // ustaw: format SPI, CPOL = 0, CPHA = 0,SCR=15
    SSPCPSR = 0x2;                  // preskaler; podzia PCLK przez 2
    SSPCR1 = 0x02;                         // SPI1 Master, SSP enabled
    for ( i = 0; i < 8; i++ ) SSPDR;               // zerowanie RxFIFO
    return;
}

//--------------------- podprogram nadawania i odbioru bajtu przez SPI
BYTE SPI_Byte(BYTE tx_byte) {
  SSPDR=tx_byte;                        // wysanie bajtu do nadawania
  do { } while ( SSPSR & 0x10  );         // sprawdzanie koca odbioru
  return SSPDR;                            // odczyt odebranych danych
}

// ------------- Podprogram nadawania i odbioru cigu bajtw przez SPI
void SPI_Data( BYTE *receive,BYTE *send, WORD Length ) {
  DWORD i;
  BYTE  *tmp = receive;
  for ( i = 0; i < Length; i++ ) {
    if(send==0)     *(receive++) = SPI_Byte(0xFF);     // tylko odczyt
    if(receive==0)  SPI_Byte(*(send++));            // tylko nadawanie
    if ( (send !=0) &&  (receive!=0)) 
                                   *(receive++) = SPI_Byte(*(send++));
  }
  receive=tmp;      // ustaw wskanik na pocztek odebranej informacji
  return;
}