/*********************************************************************
I2C_simple.c                         (obsuga I2C bez pomocy przerwa)

Komunikacja z ukadem PCF8575 za pomoc interfejsu I2C.
Do ukadu PCF8574 wysyane s kolejne wartoci 0,1,2,3... Te same
wartoci po odebraniu wywietlane s na diodach LED. Kontroler I2C
procesora '213x/4x obsugiwany jest krok po kroku bez korzystania z
systemu przerwa.
*********************************************************************/
#include <LPC213x.H>      
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

#define  NBit_LEDs 16                            // pooenie diod LED

void delay(DWORD x);                  // prototyp opnienia czasowego
void i2c_init( void);                    // inicjowanie kontrolera I2C
void i2c_write(BYTE x);                      // zapis bajtu do PCF8575
BYTE i2c_read (void);                        // odczyt bajtu z PCF8575

//--------------------------------------------------------------------
main(void) {
  BYTE out,in;                             //wysyane i odbierane dane

  IODIR1 = 0xFF << NBit_LEDs;       // piny P1.16...P1.23 jako wyjcie
  i2c_init();

  while(1) {       
      out++;                                     // dane do wysania
     //out=0xAA;                                     // dane do wysania
     i2c_write( out );                                    // wysyanie
     delay(0xfffff);                                       // poczekaj
     in = i2c_read ();                                       // odbir
     IOSET1 = 0xFF << NBit_LEDs;                 // zgaszenie diod LED
     IOCLR1 =  in  << NBit_LEDs;     // wywietlanie odebranych danych
     delay(0xfffff);                                       // poczekaj
  }
}

void i2c_init( void) { // inicjowanie I2C ----------------------------
    PINSEL0    = 0x50;       // ustawienie funkcji pinw GPIO jako I2C
    I2C0SCLH   = 0x0F;                      // czstotliwo pracy I2C
    I2C0SCLL   = 0x0F;                      // czstotliwo pracy I2C
    I2C0CONCLR = 0xFF;                           // zerowanie flag I2C
    I2C0CONSET = 0x40;                             // odblokowanie I2C
}

void i2c_write( BYTE x) { // zapis bajtu przez I2C -------------------
    I2C0CONSET = 0x20;                     // ustaw warunek startu STA

    while(!(I2C0CONSET&0x08));       // czekaj na potwierdzenie startu
    I2C0CONCLR = 0x20;                          // usu warunek startu
    I2C0DAT = 0x40;          // wylij adres z bitem zapisu (R/~W = 0)
    I2C0CONCLR = 0x08;                  // wyzeruj flag potwierdzenia
    while(!(I2C0CONSET&0x08));       // czekaj na potwierdzenie adresu

    I2C0DAT = x;                                      // wylij bajt x
    I2C0CONCLR = 0x08;                  // wyzeruj flag potwierdzenia
    while(!(I2C0CONSET&0x08));     // czekaj na potwierdzenie wysania

    I2C0CONSET = 0x10;                      // ustaw warunek stopu STO
    I2C0CONCLR = 0x08;                  // wyzeruj flag potwierdzenia
}

BYTE i2c_read (void) { // odczyt bajtu przez I2C ---------------------
    BYTE x;
    I2C0CONSET = 0x20;                     // ustaw warunek startu STA
    while(!(I2C0CONSET&0x08));      // czekamy na potwierdzenie startu

    I2C0CONCLR = 0x20;                          // usu warunek startu
    I2C0DAT = 0x41;       // wylij adres  w trybie odczytu (R/~W = 1)
    I2C0CONCLR = 0x08;                  // wyzeruj flag potwierdzenia
    while(!(I2C0CONSET&0x08));       // czekaj na potwierdzenie adresu

    I2C0CONCLR = 0x04;// wyczamy potwierdzanie danych przychodzcych
    I2C0CONCLR = 0x08;                  // wyzeruj flag potwierdzenia
    while(!(I2C0CONSET&0x08));                // czekaj na bajt danych

    x = I2C0DAT;                                 // zapisz bajt danych
    I2C0CONSET = 0x10;                       //ustaw warunek stopu STO
    I2C0CONCLR = 0x08;                  // wyzeruj flag potwierdzenia
    return x;                                        // odczytane dane
}

void delay(DWORD x) { // opnienie czasowe --------------------------
  DWORD i;
  for(i=0;i<x;i++);
}
