/*********************************************************************
IAP.c                                    (programowanie pamici Flash)
                                 ... In Aplication Programamming (IAP)

Program pozwala zapisa blok danych data do pamici Flash procesora 
pod wskazany adres. Zapisy do pamici Flash odbywaj sie sektorami. 
Procesory '214x posiadaj sektory o rozmiarach 4 lub 32KBajty. Zapis
 wymaga najpierw skasowania caego sektora. Powoduje to wypenienie 
 sektora bajtami 0xFF. Zapisy do pamici odbywaj si blokami 
 256, 512,1024 lub 4096Bajtw. Adres w pamici Flash musi by 
 zlokalizowany z dokadnoci do 256 bajtw (adres 0xXXX00). Odczyt
 z pamici Flash odbywa si za pomoc wskanika z ustawionym adresem.
Program wykorzystuj podprogramy zapisane w pamici Boot Loader.

Program transmituje znaki z parametrami:
          38400 bps, 8-N-1 (8-bitw, N- bez parzystoci, 1 bit stopu).
*********************************************************************/
#include <LPC214x.H>       
#include <stdio.h>
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "UART.h"

#define FLASH_addr   0x7c000L
#define XTAL         12000000                   // oscylator procesora

typedef struct  {  DWORD cmd;  DWORD prm[4]; }  IAP_PRM;  // polecenia
                                            //... IAP do programowania
// definicja wskanika do funkcji w obszarze Boot Loader -------------
typedef void  (*IAP)( IAP_PRM *param, DWORD *result);

void pll_start (void)  {    // odblokowanie dziaania PLL  full speed
     PLL0CON=0x3;                                         // wcz PLL
     PLL0FEED= 0xAA;
     PLL0FEED= 0x55;
}

void pll_stop(void)  {       // zablokowanie dziaania PLL  low speed
   PLL0CON=0x1;                                          // wycz PLL
   PLL0FEED= 0xAA;
   PLL0FEED= 0x55;
}

DWORD get_secnum (DWORD addr)  {    // zamiana adresu na numer sektora
  DWORD n;

  n = ( addr >> 12) & 0x7F;                                                                                          
  if (n >= (0x78000 >> 12))       n -= 98;   //  wysokie sektory 4kB
  else if (n >= (0x8000 >> 12))   n  = (n >> 3) + 7;   // sektory 32kB
                          
  return n;                                           // numer sektora
}

// Kasowanie sektorw pomidzy adresami start i end ------------------
// Poprawne zakoczenie programu zwraca warto 0.
DWORD erase (DWORD start, DWORD end)  {
  IAP_PRM  iap;                   // parametry dla polecenia kasowania
                                               // rzutowanie wskanika
  IAP iap_entry= ( void  (*)( IAP_PRM *, DWORD *)) 0x7FFFFFF1;

  DWORD result[16];                  // wyniki wykonania polecenia IAP
  DWORD save_VicInt;                    // przechowanie stanu przerwa

  save_VicInt = VICIntEnable;           //  skadowanie stanu przerwa
  VICIntEnClr = 0xFFFFFFFF;                   // zablokowanie przerwa
  pll_stop();                         // IAP wymagaj zablokowania PLL

  iap.cmd = 50;      // polecenie IAP przygotowujce sektory do zapisu
  iap.prm[0] = get_secnum (start);                  // sektor startowy
  iap.prm[1] = get_secnum (end);                     // sektor kocowy
  iap_entry (&iap, result);                 // wywoanie procedury IAP
  if (result[0])  goto fin;                       // skok gdy s bdy

  iap.cmd = 52;                        // polecenie IAP: kasuj sektory
  iap.prm[0] = get_secnum (start);                  // sektor startowy
  iap.prm[1] = get_secnum (end);                     // sektor kocowy
  iap.prm[2] = XTAL/1000;                     // zegar procesora (kHz)
  iap_entry (&iap, result);                 // wywoanie procedury IAP
fin:
  pll_start();                                           // PLL dziaa

  VICIntEnable = save_VicInt;     // stan przerwa przed zablokowaniem
  return (result[0]);                      // 0  poprawne zakoczenie
}

// Programowanie bloku danych data w miejscu flash_addr, o rozmiarze
// 256, 512, 1024 lub 4089 bajtw. Adres musi by umiejscowiony z 
// dokadnoci do 256 bajtw. Poprawne zakoczenie programu zwraca
// warto 0. --------------------------------------------------------
DWORD program (DWORD flash_addr, void *data, DWORD size)  {
  IAP_PRM  iap;                                // parametry komend IAP
                                               // rzutowanie wskanika
  IAP iap_entry= ( void  (*)( IAP_PRM *, DWORD *)) 0x7FFFFFF1;
  DWORD result[16];                  // wyniki wykonania polecenia IAP
  DWORD save_VicInt;                    // przechowanie stanu przerwa

  save_VicInt = VICIntEnable;            // skadowanie stanu przerwa
  VICIntEnClr = 0xFFFFFFFF;                   // zablokowanie przerwa
  
  pll_stop();                         // IAP wymagaj zablokowania PLL

  iap.cmd = 50;      // polecenie IAP przygotowujce sektory do zapisu
  iap.prm[0] = get_secnum (flash_addr);             // sektor startowy
  iap.prm[1] = iap.prm[0];                           // sektor kocowy
  iap_entry (&iap, result);                 // wywoanie procedury IAP
  if (result[0])  goto fin;                       // skok gdy s bdy

  iap.cmd = 51;                  // polecenie IAP: kopiuj RAM do Flash
  iap.prm[0] = (DWORD) flash_addr;            // adres w pamici Flash
  iap.prm[1] = (DWORD) data;                  // rdo danych (adres)
  iap.prm[2] = size;                           // rozmiar bloku danych
  iap.prm[3] = XTAL/1000;                     // zegar procesora (kHz)
  (*iap_entry) (&iap, result); //wywoanie procedury IAP (inny sposb)
fin:
  pll_start();                                     // odblokowanie PLL
  VICIntEnable = save_VicInt;                 // odblokowanie przerwa
  return (result[0]);                      // 0  poprawne zakoczenie
}

//--------------------------------------------------------------------
main (void)  {

  //static  DWORD data[1024];       // przykady rnych blokw danych
  //static  WORD  data[2048];
  static  BYTE  data[4096];
  
  BYTE *flash_p,x ;          // wskanik do do odczytu z pamici Flash
  WORD i;

  UART_init;                              // przygotowanie ukadu UART
  printf("SIZE of Block= %d \r\r",sizeof (data));    // rozmiar danych
  for (i = 0; i < sizeof (data); i++)   data[i] = i;  // przygotowanie
  x=erase (FLASH_addr, FLASH_addr);
  if( x) printf("Erase Error: %u\r",x);

  x=program (FLASH_addr, data, sizeof (data));
  if( x) printf("Program Error: %u\r",x);

  flash_p= (BYTE *) FLASH_addr;           // wskanik do pamici Flash
  for (i = 0; i < sizeof (data); i++)  {    
     if( (i & 0xf) == 0) printf("Pointer: %08p  : ",  flash_p+i);
     printf("%02x ",*(flash_p+i));          // zawarto pamici Flash
     if((i & 0xf) == 0xf) printf("\r");     // co 16 bajtw nowa linia
  }   
  while (1);
}

