/*********************************************************************
melody.c                                         (generowanie melodii)

Program uywa ukadu timer T1 do generowanie melodii. Melodia
przygotowana jest w postaci zapisu nutowego: wysoko tonu
i czas trwania tonu. Do odmierzania czasu trwania nuty uywany
jest ukad timer T0.
Generowany przebieg jest dostpny na wyjciu  MAT1.0 (P0.12).

Uwaga: do linii MAT1.0 (P0.12)naley doczy suchawki o 
      opornoci wikszej od 300 Ohm.
*********************************************************************/
#include <LPC214x.H>
#include "inttypes.h"             // tu zdefiniowano BYTE, WORD i DWORD

#define _PCLK 15000000                // dla kwarcu 12MHz i VPBDIV = 4

// czas trwania nuty wyraony jest w "semkach" (1/8)-----------------
#define Whole     8
#define Half      4
#define Quarter   2
#define Eighth    1

// czstotliwoci nut [Hz]--------------------------------------------
#define FreqC   523
#define FreqD   587
#define FreqE   659
#define FreqF   698
#define FreqG   784
#define FreqA   880
#define FreqB   988

// okresy nut w tykniciach PLCK -------------------------------------
#define NoteC  _PCLK / FreqC /2
#define NoteD  _PCLK / FreqD /2
#define NoteE  _PCLK / FreqE /2
#define NoteF  _PCLK / FreqF /2
#define NoteG  _PCLK / FreqG /2
#define NoteA  _PCLK / FreqA /2
#define NoteH  _PCLK / FreqH /2
#define Silence 0

// nutowy zapis melodii ----------------------------------------------
const  WORD Notes[][2] = {       
{NoteE,Quarter}, { NoteE,Quarter}, { NoteF,Quarter}, { NoteG,Quarter},
{NoteG,Quarter}, { NoteF,Quarter}, { NoteE,Quarter}, { NoteD,Quarter},
{NoteC,Quarter}, { NoteC,Quarter}, { NoteD,Quarter}, { NoteE,Quarter},
{NoteE,Half   }, { NoteD,Eighth }, { NoteD,Half   },   
{NoteE,Quarter}, { NoteE,Quarter}, { NoteF,Quarter}, { NoteG,Quarter},
{NoteG,Quarter}, { NoteF,Quarter}, { NoteE,Quarter}, { NoteD,Quarter},
{NoteC,Quarter}, { NoteC,Quarter}, { NoteD,Quarter}, { NoteE,Quarter},
{NoteD,Half   }, { NoteC,Eighth }, { NoteC,Half   }, {Silence,Whole },      
};

void T0isr(void) __irq;                 // prototyp funkcji przrewania

//--------------------------------------------------------------------
int main(void) {
  
  PINSEL0 = PINSEL0 | (1<<25);                     // P0.12 --> MAT1.0

  VICVectAddr0 = (unsigned)T0isr;       // adres podprogramu przerwani
  VICVectCntl0 = 0x24;        // przypisanie przerwania do szczeliny 0
  VICIntEnable |= 0x10;                    // zezwolenie na przerwanie

  T0TCR = 0x2;                   // zerowanie licznika i preskalera T0
  T0PR  =  _PCLK/8;        // jedna sma penej nuty (nuta to sekunda)
  T0MCR = 0x1;                 // wywoanie przerwania gdy T0TC==T0MR0
  T0MR0 = Whole ;                                  // czas jednej nuty
  
  T1TCR = 0x2;                   // zerowanie licznika i preskalera T1
  T1MCR = 0x2;                   // zerowanie licznika gdy T1TC==T1MR0
  T1MR0 = Silence;                                // cisza na pocztek
  T1EMR = 0x31;             // zmiana stanu (toggle) na wyjciu MAT1.0
  
  T0TCR = 0x1;                                // wczenie licznika T0
  T1TCR = 0x1;                                // wczenie licznika T1

  while(1)  PCON = 0x1;                       // przejcie w tryb Idle
}


// procedura obsugi przerwania licznika 0 ---------------------------
void T0isr (void) __irq { 

  static WORD  NoteNo = 0;     // numer aktualnie odtwarzanej nuty
  T1TCR = 0x2;                   // zerowanie licznika i preskalera T1
  T0TCR = 0x2;                   // zerowanie licznika i preskalera T0
  T1TCR = 0x1;                                // wczenie licznika T1
  T0TCR = 0x1;                                // wczenie licznika T0
  
  T0MR0 = Notes[NoteNo][1];                       // czas trwania nuty
  T1MR0 = Notes[NoteNo][0];                    // kolejna nuta melodii
  
  // numer kolejnej nuty
  NoteNo = (NoteNo+1) % (sizeof(Notes)/sizeof(WORD)/2);

  T0IR |= 0x1;              // zerowanie flagi przerwania od ukadu T0
  VICVectAddr = 0;                     // skasowanie adresu przerwania
}
