/*********************************************************************
modbus.c                               (transmisja Intel HEX i MODBUS)

Biblioteka procedur odbioru i nadawania ramek transmisji w standardzie
Intel HEX oraz w standardzie MODBUS.

Intel HEX:
 : <No_of_data_bytes>  <addr>   <last_frame> {... <data>  }  <LRC> CR 
       -2char-         -4char-    -2char-        -2char-    -2char-

MODBUS:
 : <No_of_data_bytes>  <addr>   <command> <xx>  {...<data> } <LRC> CR 
        -2char-        -2char-   -2char- -2char-    -2char-  -2char-
*********************************************************************/
#include <LPC214x.H> 
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

#include <stdio.h>
#include "SIO_intr.h"                    // definicje buforw koowych
#include "hex_cvn.h"  
#define RespTout        0xffffff           // timeout ramki transmisji

// Rozmiary tablic ---------------------------------------------------
#define DataSize         16                    // liczba bajtw danych
#define FrameSize  2*DataSize + 13                    // dugo ramki
BYTE FrameBuf[FrameSize];                 // bufor na ramk transmisji
#define SOF         ':'             // pocztek ramki (start of frame)

// sumy kontrolnej LCR dla bufora i zadanej liczby znakw-------------
BYTE lrc_calc(BYTE *buf, BYTE size) {    
  BYTE i,sum = 0;

  for(i=0;i<size;i++) sum += buf[i];
  return(-sum);     // suma kontrolna to suma bajtw z ujemnym znakiem
}

// Przegldanie bufora odebranych znakw w ---------------------------
// ...poszukiwaniu pocztku ramki. Zwraca SOF (warto rna od 0).---
BYTE search4head(void)  {
  BYTE c;

  while(RX_Count() > 0) {
    if( (c=getkey())== SOF) return c;
  }
  return 0 ;
}

// Zpaisawanie ramki do bufora, Kocem ramki jest znak CR.------------
// Drugie ograniczenie koca ramki to liczba bajtw: 
// liczba bajtw danych * 2 +12. Jeli nie pojawi si koniec ramki
// w zadanym czasie timeout to dugo odebranej ramki bdzie 0.
// Procedura zwraca liczb odebranych znakw.-------------------------
BYTE FrameRecord(BYTE *buf, DWORD timeout) {
 BYTE c,i=0;
 DWORD tleft;

 tleft= timeout;                           // ustawienie czasu timeout
 do {      // powtarzane do odebrania CR lub wyczerpania czasu timeout
    if(RX_Count() > 0) {
       c = getkey();
       buf[i++] = c;                     
       if (i>1  &&  i>2*Hex2Byte(buf)+12 ) return (i-1);      // limit
       if(c == '\r') return (i-1);                 // odebrano znak CR
       if(i > FrameSize) flush_RX();     // przekroczono rozmiar ramki
     }                
     tleft--;                              // odliczanie czasu timeout
 } while (tleft != 0);
 buf[0] = '\0';                           // przekroczony  limit czasu
 return (0);
}

// Pobranie ramki z odzykiwaniem pl ramki. --------------------------
// Pola maj dugo bajtu:
//   Size - liczba bajtw danych 
//   Addr - adres 
//   Cmd  - polecenie (komenda)
//   xx   - dadatkowy bajt (extra bajt)
//   Data[Size] - tablicz bajtw danych
// Uzyta jest zmienna globalna FrameBuf.
// Program zwraca warto 1 gdy odebrano poprawn ramk.--------------
BYTE GetFrame(BYTE *Size, BYTE *Addr, BYTE *Cmd, BYTE *xx, BYTE *Data)
{
  BYTE len,cnt=0,i=0;

  len = FrameRecord(FrameBuf,RespTout);              // pobranie ramki
  if(len < 10) return(0);                 //  N+N+A+A+C+C+x+x+S+S = 10
  do {
    FrameBuf[cnt++] =Hex2Byte(&FrameBuf[i]);  // konwersja HEX na bajt
    i +=2;
  } while(i < len);
 
  *Size = cnt  = FrameBuf[0];                 //  liczba bajtw danych
  *Addr        = FrameBuf[1];                                 // adres
  *Cmd         = FrameBuf[2];                   // polecenie (komenda)
  *xx          = FrameBuf[3];                        // dodatkowy bajt
  if( lrc_calc( FrameBuf, cnt+5 ) !=0 )  return(0);  // kontrola ramki
  if(cnt == 0)  { Data[0] = 0 ; return(1);}             // brak danych
  for(i=0;i<cnt;i++) { Data[i] = FrameBuf[i+4];}  // kopiowanie danych
  return(1);              // TRUE - odebrano poprawn ramk transmisji
}

// Wysanie ramki.----------------------------------------------------
// Pola ramki maj dugo bajtu:
//   Size - liczba bajtw danych 
//   Addr - adres 
//   Cmd  - polecenie (komenda)
//   xx   - dadatkowy bajt (extra bajt)
//   Data[Size] - tablicz bajtw danych    
// Uzyta jest zmienna globalna FrameBuf. -----------------------------
void PutFrame( BYTE Size, BYTE Addr, BYTE Cmd, BYTE xx, BYTE *Data) {  
  BYTE lrc,i,cnt;

  FrameBuf[0] = Size;                          // liczba bajtw danych
  FrameBuf[1] = Addr;                                         // adres
  FrameBuf[2] = Cmd;                        // komenda - polecenie Cmd
  FrameBuf[3] = xx;                     // dodatkowy bajt (extra byte)
  if( Size > 0) for(i=0;i<Size;i++)
                      FrameBuf[i+4] = Data[i];    // kopiowanie danych

  lrc = lrc_calc(FrameBuf,Size + 4);                 // wyliczenie LRC

  FrameBuf[0] = SOF;                     // zapisz znak pocztku ramki
  Byte2Hex( Size,&FrameBuf[1]);    // dopisz liczb bajtw danych Size
  Byte2Hex( Addr,&FrameBuf[3]);                   // dopisz adres Addr
  Byte2Hex( Cmd ,&FrameBuf[5]);                  // dopisz komend Cmd
  Byte2Hex( xx  ,&FrameBuf[7]);    // dopisz dodatkowy bajt xx (extra)

  cnt = 9;
  if( Size > 0 )for(i=0;i<Size;i++) {              // dopisz dane Data
                    Byte2Hex( Data[i],&FrameBuf[cnt++]); cnt++;
  }
 
  Byte2Hex( lrc,&FrameBuf[cnt++]);    // wylicz LRC i zapisz do bufora
  FrameBuf[++cnt] = '\r';                            // dopisz znak CR
  FrameBuf[++cnt] = 0;                      // dopisz znak koca ramki
  puts((const char *)FrameBuf);                        // ramka gotowa --> wysyanie
}
